@@ -6,7 +6,7 @@
 # Change 1..1 below to 1..last_test_to_print .
 # (It may become useful if the test is moved to ./t subdirectory.)
 
-BEGIN { $| = 1; print "1..36\n"; }
+BEGIN { $| = 1; print "1..45\n"; }
 END {print "not ok 1\n" unless $loaded;}
 use Randomize;
 $loaded = 1;
@@ -327,6 +327,57 @@ if (defined $r) {
 }
 
 
+$r = Randomize->new(
+  [{Field => 'ABCD',
+    Values => ['A'..'D']},
+   {Field => 'Conditional',
+    Values => [{Precondition => "<<ABCD>> eq 'A'",
+                Alternatives => [{Data => ['Aa'..'Ab'],
+                                  Weight => 3},
+                                 {Data => ['Ac'],
+                                  Weight => 1}],
+                Retry_If     => ["<<Conditional>> eq 'Aa'"]},
+               {Precondition => "<<ABCD>> eq 'B'",
+                Alternatives => ['Ba','Bb']},
+               {Precondition => 'DEFAULT',
+                Alternatives => ['CD']}],
+    Retry_If => ["<<ABCD>> eq 'A' && <<Conditional>> eq 'Ab'"]
+   }]);
+
+print 'not ' unless defined $r;
+print "ok 22\n";
+print $Randomize::errmsg;
+
+if (defined $r) {
+  my $failed = 0;
+  for (1..20) {
+    my $thing = $r->generate(ABCD => 'A');
+    if ($thing->{Conditional} ne 'Ac') {
+      $failed = 1;
+      last;
+    }
+  }
+  print 'not ' if $failed;
+  print "ok 23\n";
+
+  my $thing = eval {$r->generate(ABCD => 'A', Conditional => 'Aa')};
+  if (defined $thing || $@ !~ /violates the Retry_If/) {
+    print "not ok 24\n";
+  }
+  else {
+    print "ok 24\n";
+  }
+
+  $thing = eval {$r->generate(ABCD => 'A', Conditional => 'Ab')};
+  if (defined $thing || $@ !~ /violates the Retry_If/) {
+    print "not ok 25\n";
+  }
+  else {
+    print "ok 25\n";
+  }
+}
+
+
 # Now, a few error cases to make sure I get the right error messages
 
 # Missing "Field"
@@ -337,7 +388,7 @@ $r = Randomize->new(
 
 print 'not '
   if defined $r || $Randomize::errmsg !~ /doesn't contain a field name/;
-print "ok 22\n";
+print "ok 26\n";
 
 # Missing "Values"
 $r = Randomize->new(
@@ -346,7 +397,7 @@ $r = Randomize->new(
 
 print 'not '
   if defined $r || $Randomize::errmsg !~ /doesn't have a Values field/;
-print "ok 23\n";
+print "ok 27\n";
 
 # Missing "Data" in "Values"
 $r = Randomize->new(
@@ -356,7 +407,7 @@ $r = Randomize->new(
 
 print 'not '
   if defined $r || $Randomize::errmsg !~ /doesn't contain a Data element/;
-print "ok 24\n";
+print "ok 28\n";
 
 # Missing "Weight" in "Values"
 $r = Randomize->new(
@@ -366,7 +417,7 @@ $r = Randomize->new(
 
 print 'not '
   if defined $r || $Randomize::errmsg !~ /doesn't contain a Weight element/;
-print "ok 25\n";
+print "ok 29\n";
 
 # Bogus "Data" in "Values"
 $r = Randomize->new(
@@ -376,7 +427,7 @@ $r = Randomize->new(
 
 print 'not '
   if defined $r || $Randomize::errmsg !~ /Data element isn't an array ref/;
-print "ok 26\n";
+print "ok 30\n";
 
 # Bogus "Weight" in "Values"
 $r = Randomize->new(
@@ -386,7 +437,7 @@ $r = Randomize->new(
 
 print 'not '
   if defined $r || $Randomize::errmsg !~ /Weight element isn't a positive integer/;
-print "ok 27\n";
+print "ok 31\n";
 
 
 # No Precondition
@@ -397,7 +448,7 @@ $r = Randomize->new(
 
 print 'not '
   if defined $r || $Randomize::errmsg !~ /No precondition/;
-print "ok 28\n";
+print "ok 32\n";
 
 
 # No Alternatives
@@ -408,7 +459,7 @@ $r = Randomize->new(
 
 print 'not '
   if defined $r || $Randomize::errmsg !~ /contain a Data element/;
-print "ok 29\n";
+print "ok 33\n";
 
 
 # No Data in alternative
@@ -421,7 +472,7 @@ $r = Randomize->new(
 
 print 'not '
   if defined $r || $Randomize::errmsg !~ /doesn't contain a Data element/;
-print "ok 30\n";
+print "ok 34\n";
 
 
 # No Weight in alternative
@@ -434,7 +485,7 @@ $r = Randomize->new(
 
 print 'not '
   if defined $r || $Randomize::errmsg !~ /doesn't contain a Weight element/;
-print "ok 31\n";
+print "ok 35\n";
 
 
 # Bogus Data in alternative
@@ -447,7 +498,7 @@ $r = Randomize->new(
 
 print 'not '
   if defined $r || $Randomize::errmsg !~ /isn't an array ref/;
-print "ok 32\n";
+print "ok 36\n";
 
 
 # Bogus Weight in alternative
@@ -460,7 +511,7 @@ $r = Randomize->new(
 
 print 'not '
   if defined $r || $Randomize::errmsg !~ /isn't a positive integer/;
-print STDERR "ok 33\n";
+print "ok 37\n";
 
 
 # Gonna test out the debug directive a bit
@@ -474,7 +525,7 @@ $r = Randomize->new(
                 Retry_If     => ['<<AB>> eq $main::old_ab']}]}]);
 
 print 'not ' unless defined $r && -e 'test.pl.code';
-print "ok 34\n";
+print "ok 38\n";
 unlink 'test.pl.code';
 
 
@@ -488,7 +539,7 @@ $r = Randomize->new(
                 Retry_If     => ['<<AB>> eq $main::old_ab']}]}]);
 
 print 'not ' unless defined $r && -e 'Randomize.code';
-print "ok 35\n";
+print "ok 39\n";
 
 
 $r = Randomize->new(
@@ -500,7 +551,151 @@ $r = Randomize->new(
                 Retry_If     => ['<<AB>> eq $main::old_ab']}]}]);
 
 print 'not ' if defined $r || $Randomize::errmsg !~ /Syntax error/;
-print "ok 36\n";
+print "ok 40\n";
+
+
+
+# Now, test the permutations counter
+
+my $testno;
+
+
+$testno = 41;
+$r = Randomize->new(
+  [{Field => 'ABCD',
+    Values => ['A'..'D']},
+   {Field => 'Conditional',
+    Values => [{Precondition => "<<ABCD>> eq 'A'",
+                Alternatives => [{Data => ['Aa'..'Aj'],
+                                  Weight => 1},
+                                 {Data => ['Ak','Al'],
+                                  Weight => 10}]},
+               {Precondition => "<<ABCD>> eq 'B'",
+                Alternatives => ['Ba','Bb']},
+               {Precondition => 'DEFAULT',
+                Alternatives => ['CD']}]
+   }]);
+if ($r) {
+  my $p = $r->permutations();
+  if ($p == 16) {
+    print "ok $testno\n";
+  }
+  else {
+    print "not ok $testno ($p)\n";
+  }
+}
+else {
+  print "not ok $testno ($Randomize::errmsg)\n";
+}
+
+
+$testno++;
+$r = Randomize->new(
+  [{Field => 'ABCD',
+    Values => ['A'..'D']},
+   {Field => 'Conditional',
+    Values => [{Precondition => "<<ABCD>> eq 'A'",
+                Alternatives => [{Data => ['Aa'..'Aj'],
+                                  Weight => 1},
+                                 {Data => ['Ak','Al'],
+                                  Weight => 10}],
+                Retry_If     => ["<<Conditional>> eq 'Al'"]},
+               {Precondition => "<<ABCD>> eq 'B'",
+                Alternatives => ['Ba','Bb']},
+               {Precondition => 'DEFAULT',
+                Alternatives => ['CD']}]
+   }]);
+if ($r) {
+  my $p = $r->permutations();
+  if ($p == 15) {
+    print "ok $testno\n";
+  }
+  else {
+    print "not ok $testno ($p)\n";
+  }
+}
+else {
+  print "not ok $testno ($Randomize::errmsg)\n";
+}
+
+
+$testno++;
+$r = Randomize->new(
+  [{Field => 'ABCD',
+    Values => ['A'..'D']},
+   {Field => 'Conditional',
+    Values => [{Precondition => "<<ABCD>> eq 'A'",
+                Alternatives => [{Data => ['Aa'..'Aj'],
+                                  Weight => 1},
+                                 {Data => ['Ak','Al'],
+                                  Weight => 10}]},
+               {Precondition => "<<ABCD>> eq 'B'",
+                Alternatives => ['Ba','Bb']},
+               {Precondition => 'DEFAULT',
+                Alternatives => ['CD']}],
+    Retry_If => ["<<ABCD>> eq 'A' && <<Conditional>> eq 'Ak'"]
+   }]);
+if ($r) {
+  my $p = $r->permutations();
+  if ($p == 15) {
+    print "ok $testno\n";
+  }
+  else {
+    print "not ok $testno ($p)\n";
+  }
+}
+else {
+  print "not ok $testno ($Randomize::errmsg)\n";
+}
+
+
+$testno++;
+$r = Randomize->new(
+  [{Field => 'ABCD',
+    Values => ['A'..'D']},
+   {Field => 'Conditional',
+    Values => [{Precondition => "<<ABCD>> eq 'A'",
+                Alternatives => [{Data => ['Aa'..'Aj'],
+                                  Weight => 1},
+                                 {Data => ['Ak','Al'],
+                                  Weight => 10}],
+                Retry_If     => ["<<Conditional>> eq 'Al'"]},
+               {Precondition => "<<ABCD>> eq 'B'",
+                Alternatives => ['Ba','Bb']},
+               {Precondition => 'DEFAULT',
+                Alternatives => ['CD']}],
+    Retry_If => ["<<ABCD>> eq 'A' && <<Conditional>> eq 'Ak'"]
+   }]);
+if ($r) {
+  my $p = $r->permutations();
+  if ($p == 14) {
+    print "ok $testno\n";
+  }
+  else {
+    print "not ok $testno ($p)\n";
+  }
+
+  $testno++;
+  # See if it generates the right number of permutations when I specify
+  # a value for one of the fields.
+  $p = $r->permutations(ABCD => 'B');
+  if ($p == 2) {
+    print "ok $testno\n";
+  }
+  else {
+    print "not ok $testno ($p)\n";
+  }
+}
+else {
+  print "not ok $testno ($Randomize::errmsg)\n";
+}
+
+
+
+
+
+
+
 
 if (@slop_failures) {
   my $tcs = 'testcase';
